#include <avr/io.h>		// for the input/output register
#include <util/delay.h>
#include <avr/interrupt.h> 
//For the serial port

#define CPU_FREQ        16000000L       // Assume a CPU frequency of 16Mhz
#define PRESCALER 1024
#define NB_TICK 6400

void init_timer()
{
  TCCR1A = 0x81; 
  TCCR1B = 0x05;
}

void init_serial(int speed)
{
  /* Set baud rate */
  UBRR0 = CPU_FREQ/(((unsigned long int)speed)<<4)-1;

  /* Enable transmitter & receiver & enable interreput on RX */
  UCSR0B = 0b01011000;

  /* Set 8 bits character and 1 stop bit */
  UCSR0C = (1<<UCSZ01 | 1<<UCSZ00);

  /* Set off UART baud doubler */
  UCSR0A &= ~(1 << U2X0);
}

void send_serial(unsigned char c)
{
  loop_until_bit_is_set(UCSR0A, UDRE0);
  UDR0 = c;
}

unsigned char get_serial(void)
{
  loop_until_bit_is_set(UCSR0A, RXC0);
  return UDR0;
}

// For the AD converter
void ad_init(unsigned char channel)   
{   
  ADCSRA|=(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);   
  ADMUX|=(1<<REFS0)|(1<<ADLAR);
  ADMUX=(ADMUX&0xf0)|channel;   
  ADCSRA|=(1<<ADEN);
}   

unsigned int ad_sample(void)
{
  ADCSRA|=(1<<ADSC);
  while(bit_is_set(ADCSRA, ADSC));
  return ADCH;
}

// For the I/O 
void output_init(void)
{
  DDRB |= 0xff; // PB0 et PB5 en sortie => 22
  //DDRD |= 0xFF;
}

void output_set(unsigned char value)
{
  if(value==0) PORTB &= 0xfd; else PORTB |= 0x01;
}

void input_init(void)
{
  DDRD &= 0xf3;  // PIN 2 et 3 en entrée
  PORTD |= 0x0c; // Activation en état haut
  DDRA &= 0xFE;
  //PORTB |= 0x00;
  PORTA |= 0x00;
  //PORTD |= 0x00;
}

unsigned char input_get(void)
{
  if ((PIND & 0x08) == 0)
    return 2;
  else
    if ((PIND & 0x04) == 0)
      return 1;
  else
    return 0;
}


void traitement()
{
  OCR1A=10;
  _delay_ms(100);
  OCR1A=(OCR1A+10)%256;
  OCR1A=10;
  _delay_ms(100);
  OCR1A=(OCR1A+10)%256;
  OCR1A=10;
  _delay_ms(100);
  OCR1A=(OCR1A+10)%256;
  OCR1A=10;
  _delay_ms(100);
  OCR1A=(OCR1A+10)%256;
  OCR1A=10;
  _delay_ms(100);
  OCR1A=(OCR1A+10)%256;
  _delay_ms(100);
  OCR1A=10;
  OCR1A=(OCR1A+10)%256;
  _delay_ms(100);
  OCR1A=10;
  OCR1A=(OCR1A+10)%256;
 
  _delay_ms(2000);
  OCR1A=0;
  
}

/*
ISR(USART0_RX_vect)
{
  unsigned char data;
  data = UDR0;
  if(data == 48)
    traitement();
}
*/

// Dummy main
int main(void)
{
  //Initialisations
  //unsigned char channel1 = 0;
  //unsigned char channel2 = 1;
  //init_serial(9600);
  //ad_init(channel1);
  //ad_init(channel2);
  output_init();
  input_init();
  init_timer();
  init_serial(9600);
  sei();
  //On allume la LED rouge : l'arduino est sous tension
  //PORTB=0x02;
  //_delay_ms(300);

  for(;;)
    {     
      if((PINA & 0x01) == 1)
	{
	  // send_serial('a');
	  //_delay_ms(2000);
	  traitement();
	}
	  
      //OCR1A = (OCR1A + 10)%256; //90 degree
      //OCR1A = (OCR1A + 170)%256;
      //_delay_ms(200);
      //960-220
      //OCR1A=100;
      //1 => super rapide
      //255 => super lent

      
      /*OCR1A=255;
	_delay_ms(1000);*/
      
      /*OCR1A=60;
	_delay_ms(1000);*/
      
      /*
      OCR1A=535;  //180 degree
      _delay_ms(200);*/
    }
  cli();
  return 0;
  
  //53 = portB 00
  //50 = portB 01
  //https://github.com/arduino/Arduino/tree/master/libraries/Servo
}
