package com.damiens.lutecia.petfun;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TimePicker;
import android.widget.Toast;

import java.util.ArrayList;

/**
 * Created by Lutécia on 06/05/2017.
 */

public class ConfigurationActivity extends AppCompatActivity {
    Button b_suppression;
    Button b_ajoutHeure;
    Button b_valider;
    TimePicker timePicker;
    String horaire;
    EditText et_poids;
    private ArrayList<String> horaires = new ArrayList<String>();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_configuration);

        b_suppression = (Button) findViewById(R.id.BoutonSuppressionHeures);
        b_ajoutHeure = (Button) findViewById(R.id.BoutonAjoutHeure);
        b_valider = (Button) findViewById(R.id.BoutonValidationConfig);
        timePicker = (TimePicker) findViewById(R.id.tpconfig);
        timePicker.setIs24HourView(true);
        et_poids = (EditText) findViewById(R.id.etpoids);

        //Récupération des préférences
        final SharedPreferences prefs = getSharedPreferences("Configuration", Context.MODE_PRIVATE);
        final SharedPreferences.Editor editor = prefs.edit();

        //Récupération du poids configuré par l'utilisateur
        et_poids.addTextChangedListener(new TextWatcher() {

            public void afterTextChanged(Editable s) {

                try {
                    //Si saisi précédemment, on s'assure de supprimer la sauvegarde précédente
                    if (prefs.getInt("Poids", -1) == -1)
                        editor.putInt("Poids", Integer.parseInt(s.toString()));
                    else {
                        editor.remove("Poids");
                        editor.apply();
                        editor.putInt("Poids", Integer.parseInt(s.toString()));
                    }
                    editor.commit();
                }
                catch (Exception e) {
                    Toast.makeText(ConfigurationActivity.this, "Format erroné", Toast.LENGTH_LONG).show();
                }

            }

            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }
        });

        //Suppression des horaires enregistrés précédemment
        b_suppression.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                int taille = prefs.getInt("Nombre d'horaires", 0);

                for(int i = 0 ; i < taille ; i++) {
                    editor.remove("Horaire" + Integer.toString(i));
                }
                editor.putInt("Nombre d'horaires", 0);
                editor.apply();
                Toast.makeText(ConfigurationActivity.this, "Suppression effectuée", Toast.LENGTH_LONG).show();
            }
        });

        //Ajout d'un horaire dans les parametres
        b_ajoutHeure.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                int hour = timePicker.getHour();
                int min = timePicker.getMinute();
                horaire = formatHoraire(hour, min);
                horaires.add(horaire);
                Toast.makeText(ConfigurationActivity.this, "Ajout effectué", Toast.LENGTH_LONG).show();
            }
        });

        b_valider.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                //Nous devons inclure la possibilité qu'une configuration a déjà été faite au préalable
                int taille = prefs.getInt("Nombre d'horaires", 0);

                for(int i = 0 ; i < (horaires.size()) ; i++) {
                    editor.putString("Horaire" + Integer.toString(i + taille), horaires.get(i));
                }
                editor.putInt("Nombre d'horaires", horaires.size() + taille);
                editor.commit();
                Toast.makeText(ConfigurationActivity.this, "Configuration enregistrée", Toast.LENGTH_LONG).show();

                //Retour au menu principal
                Intent mIntent = new Intent(ConfigurationActivity.this, MainActivity.class);
                ConfigurationActivity.this.startActivity(mIntent);
            }
        });
    }

    public String formatHoraire(int hour, int min){
        if(hour < 10)
            return "0" + Integer.toString(hour) + ":" + Integer.toString(min);
        else
            return Integer.toString(hour) + ":" + Integer.toString(min);
    }
}
