package com.damiens.lutecia.petfun.Jour;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import java.io.Serializable;
import java.util.Date;

import static android.os.Build.ID;

/**
 * Created by Lutécia on 12/04/2017.
 */

public class JoursBDD implements Serializable{
    //Variables
    private static final int VERSION_BDD = 1;
    private static final String NOM_BDD = "journees.db";

    private static final String TABLE_JOURS = "table_jours";

    private static final String COL_ID = "ID";
    private static final int NUM_COL_ID = 0;
    private static final String COL_DATE = "Date";
    private static final int NUM_COL_DATE = 1;
    private static final String COL_DOSAGE = "Dosage";
    private static final int NUM_COL_DOSAGE = 2;
    private static final String COL_FREQUENCE = "Frequence";
    private static final int NUM_COL_FREQUENCE = 3;
    private static final String COL_POIDS = "Poids";
    private static final int NUM_COL_POIDS = 4;

    private SQLiteDatabase bdd;

    private MaBaseSQLite maBaseSQLite;

    //Variables propres au singleton
    private static Context context;
    private static JoursBDD JoursBDDSingleton; //unique instance

    //Methodes

    //Méthode de restauration de contexte et création de la base de données
    public JoursBDD(Context context){
        this.context=context;
        //Création de la base de données et de sa table
        maBaseSQLite = new MaBaseSQLite(context, NOM_BDD, null, VERSION_BDD);
    }

    //Methode de recuperation de l'instance de la BDD
    public static synchronized JoursBDD getInstance(Context context){
        //Création de la base de données et de sa table
        if(JoursBDDSingleton==null) {
            //CreateJoursBDD(context);
            JoursBDDSingleton = new JoursBDD(context);
        }
        return JoursBDDSingleton;
    }

    //Méthode qui ouvre la base de données en écriture
    public void open(){
        //Ouverture de la Base de données en écriture
        bdd = maBaseSQLite.getWritableDatabase();
    }

    //Méthode qui ferme la base de données
    public void close(){
        //Fermeture de la Base de données
        bdd.close();
    }

    //Méthode qui récupère la base de données
    public SQLiteDatabase getBDD(){
        return bdd;
    }

    //Méthode qui permet d'insérer un jour dans la base
    public long insertJour(Jour jour) {
        //ContentValues fonctionne comme une HashMap
        ContentValues values = new ContentValues();
        values.put(COL_DATE, String.valueOf(jour.getDate())); //Un wrapping est necessaire
        values.put(COL_DOSAGE, jour.getDosage());
        values.put(COL_FREQUENCE, jour.getFrequence());
        values.put(COL_POIDS, jour.getPoids());
        //Insertion dans la base de données
        return bdd.insert(TABLE_JOURS, null, values);
    }

    //Méthode qui permet de mettre à jour la base
    public int updateJour(int id, Jour jour){
        ContentValues values = new ContentValues();
        values.put(COL_DATE, String.valueOf(jour.getDate())); //Un wrapping est necessaire
        values.put(COL_DOSAGE, jour.getDosage());
        values.put(COL_FREQUENCE, jour.getFrequence());
        values.put(COL_POIDS, jour.getPoids());
        //Modification dans la base de données
        return bdd.update(TABLE_JOURS, values, COL_ID + " = " + id, null);
    }

    //Méthode qui permet de supprimer un jour de la table en passant son ID en paramètre
    public int removeJourWithID(int id){
        return bdd.delete(TABLE_JOURS, COL_ID + " = " + id, null);
    }

    //Méthode qui permet la récupération d'un jour en passant sa date en paramètre
    public Jour getJourWithDate(String date){
        //Récupération du jour dans un Cursor
        Cursor c = bdd.query(TABLE_JOURS, new String[] {COL_ID, COL_DATE, COL_DOSAGE, COL_FREQUENCE, COL_POIDS}, COL_DATE + " LIKE \"" + date + "\"", null, null, null, null);
        return cursorToJour(c);
    }

    public Jour getJourWithID(int id){
        //Cursor c = bdd.query(TABLE_JOURS, new String[] {COL_ID + " LIKE \"" + Integer.toString(id) + "\"", COL_DATE, COL_DOSAGE, COL_FREQUENCE, COL_POIDS}, COL_DATE , null, null, null, null);
        Cursor c = bdd.query(TABLE_JOURS, new String[] {COL_ID , COL_DATE, COL_DOSAGE, COL_FREQUENCE, COL_POIDS}, COL_ID + " LIKE \"" + id + "\"", null, null, null, null);
        return cursorToJour(c);
    }

    //Fonction qui permet de retrouver le jour dont le curseur correspondant est passé en paramètre
    private Jour cursorToJour(Cursor c) {
        //Si vide, on retourne null
        if(c.getCount()==0)
            return null;

        //Sinon on se place sur le premier element
        c.moveToFirst();
        //Creation d'un jour
        Jour jour = new Jour();
        //On complete les champs du jour grâce au contenu du Cursor
        jour.setId(c.getInt(NUM_COL_ID));
        jour.setDate(c.getString(NUM_COL_DATE)); //Probleme car type date ?
        jour.setDosage(c.getInt(NUM_COL_DOSAGE));
        jour.setFrequence(c.getInt(NUM_COL_FREQUENCE));
        jour.setPoids(c.getInt(NUM_COL_POIDS));
        //Fermeture du Cursor
        c.close();
        //On retourne le jour
        return jour;
    }
}
