package com.damiens.lutecia.petfun;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.database.MatrixCursor;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.SimpleAdapter;
import android.widget.SimpleCursorAdapter;
import android.widget.TextView;
import android.widget.Toast;

import com.damiens.lutecia.petfun.Jour.Jour;
import com.damiens.lutecia.petfun.Jour.JoursBDD;

import org.achartengine.ChartFactory;
import org.achartengine.chart.BarChart;
import org.achartengine.model.XYMultipleSeriesDataset;
import org.achartengine.model.XYSeries;
import org.achartengine.renderer.XYMultipleSeriesRenderer;
import org.achartengine.renderer.XYSeriesRenderer;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutionException;

/**
 * Created by Lutécia on 30/04/2017.
 */

/*** Cette activité permet de visualiser les statistiques de l'animal via des graphiques ***/

public class StatistiquesActivity extends AppCompatActivity {
    //Déclaration des variables et widgets
    private View chart;
    private Button b_semaineChart;
    private Button b_moisChart;
    ListView tab_view;

    //Récupération des préférences

    int poids;

    //Si le radio bouton lineaire est coché, la variable lineaire vaut true sinon false (histogramme dans ce cas)
    boolean lineaire = false;
    boolean tableau = false; //Si le radio bouton tableau est coché, lavariable tableau vaut true sinon false
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_statistiques);

        final SharedPreferences prefs = getSharedPreferences("Configuration", Context.MODE_PRIVATE);
        //Récupération du poids d'une gamelle enregistré dans les préférences
        //Si l'utilisateur ne l'a pas renseigné, le nombre de gamelle sera affiché
        if(prefs.getInt("Poids",-1) == -1)
            poids = 1;
        else
            poids = prefs.getInt("Poids",-1);

        //Récupération de l'instance de la base de données
        final JoursBDD joursBDD = JoursBDD.getInstance(this);

        /*Jour j = new Jour("22/08/1995", 25, 2, 10);
        Jour jFromBdd = joursBDD.getJourWithDate(j.getDate());
        jFromBdd = joursBDD.getJourWithDate(j.getDate());
        if(jFromBdd != null) {
            Toast.makeText(this, jFromBdd.toString(), Toast.LENGTH_LONG).show();
            //Supression du jour
            joursBDD.removeJourWithID(jFromBdd.getId());
        }*/

        //Association des boutons et de leur ID
        b_semaineChart = (Button) findViewById(R.id.semaineChartBouton);
        b_moisChart = (Button) findViewById(R.id.moisChartBouton);

        //Ouverture des graphiques ou affichage des tableaux
        b_semaineChart.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                //Recuperation de la date du jour
                Calendar c = Calendar.getInstance();
                int an = c.get(Calendar.YEAR);
                int mois = (c.get(Calendar.MONTH)+1)%12;
                int jour = c.get(Calendar.DAY_OF_MONTH);
                //on récupère le numéro du jour dans la semaine
                String d_jour = Integer.toString(jour) + "/" + Integer.toString(mois) + "/" + Integer.toString(an);

                //Recuperation des 7 derniers jours de la base de données dans un tableau
                Jour j;
                try {
                    j = joursBDD.getJourWithDate(d_jour);
                    String[] dates = new String[7];
                    int gamelles_videes[] = new int[7];
                    for (int i = 0; i < 7; i++) {
                        dates[i] = (joursBDD.getJourWithID((j.getId()) - i)).getDate();
                        gamelles_videes[i] = poids * ((joursBDD.getJourWithID((j.getId()) - i)).getFrequence());
                    }

                    if (!tableau) {
                        int[] x = {0, 1, 2, 3, 4, 5, 6};
                        createChart(x, dates, gamelles_videes);
                    } else
                        createTableau(dates, gamelles_videes);
                }
                catch (NullPointerException e) {
                    Toast.makeText(StatistiquesActivity.this, "Mise à jour nécessaire", Toast.LENGTH_LONG).show();
                }
            }
        });

        b_moisChart.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                //Recuperation de la date du jour
                Calendar c = Calendar.getInstance();
                int an = c.get(Calendar.YEAR);
                int mois = (c.get(Calendar.MONTH)+1)%12;
                int jour = c.get(Calendar.DAY_OF_MONTH);
                //on récupère le numéro du jour dans la semaine
                String d_jour = Integer.toString(jour) + "/" + Integer.toString(mois) + "/" + Integer.toString(an);

                //Recuperation des 30 derniers jours de la base de données dans un tableau
                Jour j;
                try {
                    j = joursBDD.getJourWithDate(d_jour);
                    String[] dates = new String[30];
                    int gamelles_videes[] = new int[30];
                    for (int i = 0; i < 30; i++) {
                        dates[i] = (joursBDD.getJourWithID((j.getId()) - i)).getDate();
                        gamelles_videes[i] = poids * ((joursBDD.getJourWithID((j.getId()) - i)).getFrequence());
                    }

                    if (!tableau) {
                        int[] x = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29};
                        createChart(x, dates, gamelles_videes);
                    } else
                        createTableau(dates, gamelles_videes);
                }
                catch (NullPointerException e) {
                    Toast.makeText(StatistiquesActivity.this, "Mise à jour nécessaire", Toast.LENGTH_LONG).show();
                }
            }
        });

    }

    //Fonction qui permet la creation d'un graphique
    private void createChart(int[] x, String[] dates, int[] nb_gamelles_vides) {
        //Nous devons transformer le tableau nb_gamelles_vides en liste pour pouvoir trier et/ou trouver le maximum facilement
        List<Integer> l = new ArrayList<Integer>();
        for(Integer i : nb_gamelles_vides)
            l.add(i);

        // Création d'un XYSeries pours le nombre de gamelles vides
        XYSeries expenseSeries = new XYSeries("nb_gamelles_vides");
        // Ajout des données à la série de nombre de gamelles vides
        for (int i = 0; i < x.length; i++) {
            expenseSeries.add(i, nb_gamelles_vides[i]);
        }
        // Création d'un dataset pour contenir la série associée au nombre de gamelles vides
        XYMultipleSeriesDataset dataset = new XYMultipleSeriesDataset();
        // Ajout de la série "Gamelles vides" au dataset associée
        dataset.addSeries(expenseSeries);

        // Création d'un XYSeriesRenderer pour personnaliser le graphique (basique) et pour l'affichage
        XYSeriesRenderer gamelleRenderer = new XYSeriesRenderer();
        gamelleRenderer.setColor(Color.BLUE);
        gamelleRenderer.setFillPoints(true);

        gamelleRenderer.setChartValuesTextSize(20);
        gamelleRenderer.setDisplayChartValues(true);

        // Création d'un XYMultipleSeriesRenderer pour personnaliser le graphique en entier
        XYMultipleSeriesRenderer renderer = new XYMultipleSeriesRenderer();
        renderer.setXLabels(0); //Ligne importante car sans elle il y de la superposition de données
        renderer.setChartTitle("Évolution de l'alimentation de votre animal ");
        renderer.setXTitle("Dates");
        if(poids == 1)
            renderer.setYTitle("nombre de gamelles vidées");
        else
            renderer.setYTitle("Quantité de croquettes consommées en grammes");

        /***Personnalisation des graphiques***/

        // Couleur de l'axe x
        renderer.setXLabelsColor(Color.BLACK);
        // Couleur de la grille
        renderer.setGridColor(Color.GRAY);
        // Couleur des labels
        renderer.setLabelsColor(Color.BLACK);
        // Taille du titre
        renderer.setChartTitleTextSize(50);
        // Taille de l'axe du titre
        renderer.setAxisTitleTextSize(40);
        // Taille des labels du graphique
        renderer.setLabelsTextSize(20);
        // Visibilité des boutons de zoom
        renderer.setZoomButtonsVisible(true);
        // Permettre le mouvement du graphe de par et d'autres des axes x et y
        renderer.setPanEnabled(true, true);
        // Permettre de cliquer sur le graphe
        renderer.setClickEnabled(false);
        // Permettre de zoomer sur les axes x et y
        renderer.setZoomEnabled(true, true);
        // Afficher la grille des y
        renderer.setShowGridY(true);
        // Afficher la grille des x
        renderer.setShowGridX(true);
        // Permettre à la légende d'être adapté à la taille de l'écran
        renderer.setFitLegend(false);
        // Afficher les lignes de la grille
        renderer.setShowGrid(true);
        // Permettre de zoomer sur l'écran
        renderer.setZoomEnabled(false);
        // Permettre les fonctions de zoom externes
        renderer.setExternalZoomEnabled(false);
        // Formater les lignes du graphe : IMPORTANT
        renderer.setAntialiasing(true);
        // Permettre de scroller l'écran
        renderer.setInScroll(true);
        // Taille de la légende
        renderer.setLegendHeight(20);
        // Alignement de l'axe des x
        renderer.setXLabelsAlign(Paint.Align.RIGHT);
        // Alignement de l'axe des y
        renderer.setYLabelsAlign(Paint.Align.LEFT);
        // Style du texte
        //renderer.setTextTypeface("sans_serif", Typeface.NORMAL);
        // Nombre de valeurs à afficher sur l'axe des y
        renderer.setYLabels(10);
        // Valeur minimale prise par y
        renderer.setYAxisMin(Collections.min(l)-1 * poids);
        // Valeur maximale prise par y
        renderer.setYAxisMax(Collections.max(l)+ 1 *poids);
        // Valeur minimale prise par x
        renderer.setXAxisMin(-1);
        // Valeur maximale prise par x
        renderer.setXAxisMax(nb_gamelles_vides.length);
        // Espace entre deux bars (histogramme)
        renderer.setBarSpacing(0.5);
        // Couleur de l'arrière plan
        renderer.setBackgroundColor(Color.TRANSPARENT);
        // Couleur des marges
        renderer.setMarginsColor(Color.WHITE);
        renderer.setApplyBackgroundColor(true);
        renderer.setScale(2f);
        // Taille des points
        renderer.setPointSize(4f);
        // Réglage des marges (haut, gauche, bas, droit
        renderer.setMargins(new int[]{20, 10, 150, 20});

        //Affichage de nos dates sur l'axe des x
        for (int i = 0; i < x.length; i++) {
            renderer.addXTextLabel(i, dates[i]);
            //renderer.setXLabelsPadding(10);
            //renderer.setYLabelsPadding(10);
        }

        renderer.addSeriesRenderer(gamelleRenderer);

        // Affichage du graphique sur le .xml
        LinearLayout chartContainer = (LinearLayout) findViewById(R.id.chart);
        // Suppression des toutes les views avant d'afficher le graphique
        chartContainer.removeAllViews();

        // Affichage du graphique : histogramme ou graphique linéaire selon le radio Bouton sélectionné
        if(lineaire) {
            chart = ChartFactory.getLineChartView(StatistiquesActivity.this, dataset, renderer);
        }
        else {
            chart = ChartFactory.getBarChartView(StatistiquesActivity.this, dataset, renderer, BarChart.Type.DEFAULT);
        }

        // Ajout de la view au linearlayout
        chartContainer.addView(chart);
    }

    //Fonction qui permet d'afficher des tableaux
    private void createTableau(String[] dates, int[] nb_gamelles_vides) {
        tab_view = (ListView) findViewById(R.id.lv);
        TextView col1 = (TextView) findViewById(R.id.textViewCol1);
        TextView col2 = (TextView) findViewById(R.id.textViewCol2);
        String[] from = { "date", "nombre" };
        int[] to = { android.R.id.text1, android.R.id.text2 };
        ArrayList<Map<String, String>> list = buildData(dates, nb_gamelles_vides);
        SimpleAdapter adapter = new SimpleAdapter(this, list, android.R.layout.simple_list_item_2, from, to);
        tab_view.setAdapter(adapter);
    }

    //Fonction qui permet la creation d'une liste et la rempli avec nos données à afficher
    private ArrayList<Map<String, String>> buildData(String[] dates, int[] nb_gamelles_vides) {
        ArrayList<Map<String, String>> list = new ArrayList<Map<String, String>>();
        for(int i = 0 ; i < dates.length ; i++) {
            list.add(putData(dates[i], Integer.toString(nb_gamelles_vides[i]) + "g"));
        }
        return list;
    }

    //Fonction qui permet l'insertion de chaque valeur dans la hashmap
    private HashMap<String, String> putData(String date, String nombre) {
        HashMap<String, String> item = new HashMap<String, String>();
        item.put("date", date);
        item.put("nombre", nombre);
        return item;
    }

    //Fonction qui permet de changer la variable lineaire ou tableau selon le choix de l'utilisateur
    public void onRadioButtonClicked(View view) {
        // Vérification : le bouton est-il sélectionné ?
        boolean checked = ((RadioButton) view).isChecked();

        // Choix selon quel radio Bouton a été cliqué
        switch(view.getId()) {
            case R.id.xyRButton: //Graphique linéaire demandé
                if (checked) {
                    lineaire = true;
                    tableau = false;
                }
                break;
            case R.id.histoRButton: //Histogramme demandé -> par défaut
                if (checked) {
                    lineaire = false;
                    tableau = false;
                }
                break;
            case R.id.tableauRButton: //Tableau demandé
                if (checked) {
                    tableau = true;
                    lineaire = false;
                }
                break;
        }
    }
}
